/* eslint-disable no-underscore-dangle */
import dva from 'dva';
import createLoading from 'dva-loading';
import moment from 'moment';
import persistReducer from 'redux-persist/lib/persistReducer';
import storage from 'redux-persist/lib/storage';
import { getHistory } from './services/route';
import { initApp } from './data/app';
import appModel from './models/app';
import routerConfig from './router';
import { processError } from './utils/error';
import config from './utils/config';
import './index.css';

moment.locale('zh-cn');

const persistConfig = {
  key: 'root',
  debug: process.env.NODE_ENV === 'development',
  storage,
};

// 1. Initialize
const app = dva({
  history: getHistory({
    basename: config.basename,
  }),
  onReducer: reducer => persistReducer(persistConfig, reducer),
  onError(error) {
    // eslint-disable-next-line no-console
    processError(error);
  },
});

app.model(appModel);

// 2. Plugins
app.use(createLoading({
  effects: true,
}));

// 3. Model
// app.model(require('./models/example'));

// 4. Router
app.router(routerConfig);

// 5. Start
initApp(app).then(theApp => theApp.start('#root'));
