import createBrowserHistory from 'history/createBrowserHistory';
import isString from 'lodash/isString';
import {
  push as pushAction,
  replace as replaceAction,
  go as goAction,
  goBack as goBackAction,
  goForward as goForwardAction,
} from 'react-router-redux';
import { makePath } from '../utils/helper';
import { getStore } from '../data/app';

const processPath = (base, path, withContext = false) => {
  if (isString(path)) {
    return makePath(base, path, withContext);
  }
  return {
    ...path,
    pathname: makePath(base, path.pathname, withContext),
  };
};

const getHistoryBase = (history) => {
  if (history.location) {
    return history.location.pathname;
  } else {
    return location.pathname;
  }
};

let history;
let location;

export const getHistory = (...args) => {
  return history || createHistory(...args);
};
export const getLocation = () => {
  return location || {};
};
export const destroyHistory = () => {
  if (history && history.unlisten) {
    history.unlisten();
    history = null;
  }
};
export const createHistory = (...args) => {
  destroyHistory();
  history = createBrowserHistory(...args);
  // noinspection JSUnresolvedFunction
  history.unlisten = history.listen((loc) => {
    location = { ...loc };
  });
  return history;
};

export const push = (path, state) => {
  return getStore().dispatch(pushAction(processPath(getHistoryBase(history), path, false), state));
};

export const replace = (path, state) => {
  return getStore().dispatch(replaceAction(processPath(getHistoryBase(history), path, false), state));
};

export const go = (n) => {
  return getStore().dispatch(goAction(n));
};

export const goBack = () => {
  return getStore().dispatch(goBackAction());
};

export const goForward = () => {
  return getStore().dispatch(goForwardAction());
};

export const thisPush = (theThis, pathOrLoc, state) => {
  return push(pathOrLoc, state);
};

export const thisReplace = (theThis, pathOrLoc, state) => {
  return replace(pathOrLoc, state);
};

export const thisGo = (theThis, n) => {
  return go(n);
};

export const thisGoBack = () => {
  return goBack();
};

export const thisGoForward = () => {
  return goForward();
};
